/* 
 * File:   Point.h
 * Author: Kolja Junginger
 *
 * Created on February 20, 2020, 2:28 PM
 * 
 * For more details on the algorithms see 
 * "Certified Approximation Algorithms for the Fermat Point" by 
 * Kolja Junginger, Ioannis Mantas, Evanthia Papadopoulou, Martin Suderland, and Chee Yap:
 * http://www1.pub.informatik.uni-wuerzburg.de/eurocg2020/data/uploads/papers/eurocg20_paper_72.pdf
 */

#ifndef POINT_H
#define POINT_H

#include<iostream>
#include<cmath>
#include<fstream>
#include<sstream>

/* Check the PCL library as an alternative */

class Point {
public:
    Point(double a, double b);
    Point();
    double x;
    double y;
};

inline Point::Point(double a, double b)
{
    x = a;
    y = b;
}

inline Point::Point()
{
    x = 0;
    y = 0;
}

inline bool operator==(const Point& p, const Point& q) {
    return (p.x == q.x && p.y == q.y);
}

inline bool operator!=(const Point& p, const Point& q) {
    return !(p == q);
}

inline Point operator+(const Point& p, const Point& q) {
    Point sumP = Point(p.x + q.x, p.y + q.y);
    return sumP;
}

inline Point operator-(const Point& p, const Point& q) {
    Point sumP = Point(p.x - q.x, p.y - q.y);
    return sumP;
}

inline Point operator/(const Point& p, double& a) {
    Point sumP = Point(p.x/a, p.y/a);
    return sumP;
}

inline Point operator*(const Point& p, double& a) {
    Point sumP = Point(p.x * a, p.y * a);
    return sumP;
}



inline std::ostream& operator<<(std::ostream& os, Point& p) {
    return os << '(' << p.x << ',' << p.y << ")\n";
}

inline std::istream& operator>>(std::istream& is, Point& p) {
    double x, y;
    char c1, c2, c3;
    is >> c1 >> x >> c2 >> y >> c3;
    if (!is) return is;
    if (c1 != '(' || c2 != ',' || c3 != ')') {
        throw std::runtime_error("Input file has wrong format."); 
        return is;
    }
    
    p = Point(x, y);
    return is;
}

inline double normP(const Point& p) {
    return sqrt(p.x*p.x + p.y*p.y);
}

inline double dist(const Point &p1, const  Point &p2) {
    //	return sqrt(pow(p2.x - p1.x, 2) + pow(p2.y - p1.y, 2));
    return normP(p1 - p2);
}

inline int precision(double epsilon, double x){
    int dig_after  = ceil( log10( 1.0/ std::min( 0.1, epsilon) ) );
    int dig_before = ceil(log10( std::max( 1.0,std::abs(x) ) ) ) ;
    return dig_after+dig_before;
} 

inline void print_time(clock_t begin){
    /*    The function print_time(begin) shows the time since begin, defined as:
     clock_t begin = clock(); */
    
    clock_t end = clock();
    double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
    std::cout << "Elapsed time : " << elapsed_secs<< " sec"<<std::endl;
}

#endif /* POINT_H */

