/* 
 * File:   main.cpp
 * Author: Kolja Junginger
 *
 * Created on February 17, 2020, 9:30 AM
 * This file contains the 'main' function. Program execution begins and ends there.
 * 
 * For more details on the algorithms see 
 * "Certified Approximation Algorithms for the Fermat Point" by 
 * Kolja Junginger, Ioannis Mantas, Evanthia Papadopoulou, Martin Suderland, and Chee Yap:
 * http://www1.pub.informatik.uni-wuerzburg.de/eurocg2020/data/uploads/papers/eurocg20_paper_72.pdf
 */


#include "Point.h"
#include "Parser.h"
#include "weisfeld.cpp"
#include "boxsubdivison.cpp"
#include "focifermatpointtest.cpp"

int main(){       
    
    bool show_foci = false;
    
    double epsilon = pow(10, -4);
    
    string filename = "degenerate_100_1_10";
    Parser parser;
    vector<Point> foci = parser.ParsePointformat(filename);
    
    int k = foci.size();
    if (k==0){throw runtime_error("No points have been read from input file.");}
    cout<<"Number of foci: "<<k<<endl;
    
    if (show_foci) parser.PrintFoci(foci);
    
    /* --- Check if the Fermat point equals a focus --- */
    int fermatpointind = focusisfermatpoint(foci);
    
    if (fermatpointind == -1){
        
        /* --- Run our point sequence algorithm --- */
        Point fp = weisfeld(foci, epsilon);
        
        /* --- Run our box subdivision algorithm --- */
        Point FP_SUB = fermatpoint(foci,epsilon);
        
    } 
    
    return 0;
}
